@@ -1411,7 +1411,7 @@ _purifyQuotedString(NSMutableString *quotedString) {
 - (BOOL)_parseStatusResponseIntoHashMap:(NGMutableHashMap *)result_ {
   NSString            *name  = nil;
   NSMutableDictionary *flags = nil;
-  NSDictionary *d, *f;
+  NSDictionary *d;
     
   if (!_matchesString(self, "STATUS "))
     return NO;
@@ -1502,17 +1502,20 @@ _purifyQuotedString(NSMutableString *quotedString) {
 
   while (_la(self, 0) != ')') {
     NSString *key   = [self _parseQuotedString];
-    _consumeIfMatch(self, ' ');
-    NSString *value = [self _parseQuotedString];
+    _consume(self, 1); // ' '
+    NSString *value = [self _parseQuotedStringOrNIL];
 
     if (_la(self, 0) == ' ')
       _consume(self, 1);
 
-    [attributes setObject:value
-           forKey:[key lowercaseString]];
+    if (value)
+      {
+	[attributes setObject:value
+		       forKey:[key lowercaseString]];
 
-    [d setObject:[NSMutableDictionary dictionaryWithDictionary:attributes]
-       forKey:entry];
+	[d setObject:[NSMutableDictionary dictionaryWithDictionary:attributes]
+	      forKey:entry];
+      }
   }
   _consumeIfMatch(self, ')');
   _parseUntil(self, '\n');
@@ -2158,21 +2161,30 @@ static NSDictionary *_parseContentDisposition(NGImap4ResponseParser *self)
 
 static NSArray *_parseAddressStructure(NGImap4ResponseParser *self) {
   NSString *personalName, *sourceRoute, *mailboxName, *hostName;
-  
-  _consumeIfMatch(self, '(');
-  personalName = _parseBodyString(self, YES);
-  _consumeIfMatch(self, ' ');
-  sourceRoute = _parseBodyString(self, YES);
-  _consumeIfMatch(self, ' ');
-  mailboxName = _parseBodyString(self, YES);
-  _consumeIfMatch(self, ' ');
-  hostName = _parseBodyString(self, YES);
-  _consumeIfMatch(self, ')');
-  return [NSDictionary dictionaryWithObjectsAndKeys:
-                       personalName, @"personalName",
-                       sourceRoute,  @"sourceRoute",
-                       mailboxName,  @"mailboxName",
-                       hostName,     @"hostName", nil];
+
+  // Handle broken address list - generally sent by Microsoft Exchange.
+  if (_la(self, 0) == '(') {
+    _consume(self, 1);
+
+    personalName = _parseBodyString(self, YES);
+    _consumeIfMatch(self, ' ');
+    sourceRoute = _parseBodyString(self, YES);
+    _consumeIfMatch(self, ' ');
+    mailboxName = _parseBodyString(self, YES);
+    _consumeIfMatch(self, ' ');
+    hostName = _parseBodyString(self, YES);
+    _consumeIfMatch(self, ')');
+
+    return [NSDictionary dictionaryWithObjectsAndKeys:
+                           personalName, @"personalName",
+                         sourceRoute,  @"sourceRoute",
+                         mailboxName,  @"mailboxName",
+                         hostName,     @"hostName", nil];
+  }
+
+  _parseUntil(self, ')');
+
+  return [NSDictionary dictionary];
 }
 
 static NSArray *_parseParenthesizedAddressList(NGImap4ResponseParser *self) {
